==> bumpster-0.1.6.tar.gz: bumpster-0.1.6/bumpster/bump.py <==
# -*- coding: utf-8 -*-

from configparser import ConfigParser
from pathlib import Path

from .color import ColorUI
from .version import Version
from .git import Git
from .source import SourceCode
from . import __version__


#######################################
#
#   Bump Class
#
#######################################


class Bump(object):
    """
    Object which will orchestrate entire program
    """
    CONFIG_FILE = '.bumpster'
    DEFAULT_CONFIG = {
        'version': {'current': '0.1.0'},
        'options': {
            'auto_tag': False,
            'auto_commit': False,
            'auto_push': False,
            'signature': False
        },
        'file1': {
            'file': '*.py',
            'regex': '__version__\s*=\s*(?P<quote>[\'\"])(?P<version>.+)(?P=quote)',
            'style': 'semantic'
        }
    }

    def _read_file(self, f):
        return Path(f).read_text()

    def __init__(self, verbose=False, dry_run=False, defaults=False, config_file=None):
        self.ran_from = Path.cwd()
        self.verbose = verbose
        self.dryrun = dry_run
        self.defaults = defaults
        self.config_file = config_file
        self.filename = None
        self.version = None
        self.bumped_part = None

        self.config = None
        self.config_file = None
        self.auto_sign = None
        self.auto_commit = None
        self.auto_push = None
        self.signature = None
        self.source_files = []
        
        self.debug(f'   ---=== Welcome to bumpster v{__version__} ===---')
        self.debug('Verbose output selected.')
        if self.dryrun:
            self.print('--dry-run detected.  No files will be written to.', ColorUI.CYAN)
        if self.dry_run and not self.verbose:
            self.print('FYI, --dry-run without --verbose is fairly useless.', ColorUI.RED)
        if self.defaults:
            self.debug('Default config selected via --defaults.')
        
        self.debug('Investigating a project git repo.')
        self.git = Git()
        if self.git.usable:
            v = self.git.current_version
            p = self.git.distance_to_latest_tag
            b = self.git.commit_sha
            self.debug(f'Parsed git version {v}-{p}+{b}.')
        self._read_config()
        
        return

    def _read_config(self):
        """
        During the Bump __init__() this method helps read in the config.  We search 
        for a config in several locations.  Also, the user may specify default 
        config, which we load here as well.
        """
        f = self.config_file or Bump.CONFIG_FILE
        candidates = [
            Path(self.ran_from, f),
            Path(self.ran_from.parent, f),
            Path(self.git.root, f),
            Path(self.ran_from.parent.parent, f)
        ]
        
        config = ConfigParser()
        if self.defaults:
            config.read_dict(Bump.DEFAULT_CONFIG)

            file = Path(self.ran_from, Bump.CONFIG_FILE)
            config.write(open(file, "w"))
            self.config_file = file
        else:
            # Attempt to load settings from multiple locations
            for c in candidates:
                if c.exists():
                    config.read(c)
                    self.config_file = c
                    break
            else:
                raise ValueError(f'Could not find config file [{Bump.CONFIG_FILE}].')

        # Loop through config to debug it if --verbose
        for section in config.sections():
            self.debug(f'* Config section parsed: --== {section} ==--')
            for name, value in config.items(section):
                self.debug(f'* {name} = {value}')

        # CURRENT VERSION
        version = config.get('version', 'current', fallback='0.1.0')
        v = Version.parse(version)
        self.version = v
        self.debug(f'Parsed version {v.semantic} from config.')

        # OPTIONS
        self.auto_tag = config.getboolean('options', 'auto-tag', fallback=False)
        self.auto_commit = config.getboolean('options', 'auto-commit', fallback=False)
        self.auto_push = config.getboolean('options', 'auto-push', fallback=False)
        self.signature = config.getboolean('options', 'signature', fallback=False)

        o = 'auto_tag={} auto_commit={} auto_push={} signature={}'.format(
            self.auto_tag,
            self.auto_commit,
            self.auto_push,
            self.signature)
        self.debug(f'Options parsed.  {o}')


        # We need to verify sections are there, especially before we blindly remove

        # FILES TO WRITE
        sections = config.sections().copy()
        sections.remove('version')
        sections.remove('options')
        for s in sections:
            file = config.get(s, 'file', fallback=None)
            regex = config.get(s, 'regex', fallback=None)
            style = config.get(s, 'style', fallback='version')
            
            # First throw an error if need be
            if file is None or regex is None:
                self.debug(f'Skipping {s}, missing file or regex')
                continue

            # Case with *.py for all python files
            if '*' in file:
                files = self.search_for_files(file)
                for f in files:
                    u = SourceCode(self, f, regex, style)
                    self.source_files.append(u)
            # This is the normal case, 1 file per section
            else:
                u = SourceCode(self, file, regex, style)
                self.source_files.append(u)

        self.config = config
        return None

    def _persist_version(self):
        """
        Update the version in the config file then write it so we know the
        new version next time
        """
        self.debug(f'Persisting version {self.version.semantic} in config file.')
        
        self.config.set('version', 'current', self.version.semantic)
        if not self.dry_run:
            self.config.write(open(self.config_file, "w"))

        return None

    @property
    def new_release(self):
        if self.version.prerelease:
            return False

        if self.bumped_part in ['major', 'minor', 'patch']:
            self.debug('New release will trigger git tag, commit, push.')
            return True

    @property
    def tag_name(self):
        return f'v{self.version.public}'

    @property
    def config_root(self):
        return self.config_file.parent

    @property
    def dry_run(self):
        if self.dryrun:
            self.print('Skipping file write due to --dry-run', ColorUI.RED)
            return True
        return False

    def print(self, line, color=ColorUI.BLUE):
        """
        Print with fancy colors
        """

        print(f'{color}{line}{ColorUI.ENDC}')

    def debug(self, line):
        """
        Simple wrapper used to output debug messages
        for those using the flag --verbose
        """
        
        if self.verbose:
            self.print(line, ColorUI.PINK)

    def search_for_files(self, f):
        # First we look in current_dir + all subdirs
        root = self.config_root
        self.debug(f'Searching for {f} in dir {root}')
        matches = [x for x in Path(root).rglob(f)]
        if len(matches) > 0:
            return matches

    def parse_version_from_file(self, f):
        """
        Often times the only record of current_version is inside
        a source file such as __init__.py
        """

        file_txt = self._read_file(f)
        self.debug(f'Opened and read file: {f} to memory.')

        match = self.compiled_version.search(file_txt, re.MULTILINE)
        if match is None:
            raise ValueError('Could not find __version__ in __init__.py.')
        self.debug(f'Found potential version: {match.group(1)}.')

        v = Version.parse(match.group(1))
        self.debug(f'Successfully parsed version: {v.version}.')

        return v

    def bump(self, part):
        """
        This is pretty much the full process
        """

        self.bumped_part = part
        self.debug(f'Beginning bump of `{part}`...')
        self.debug('First we must save the current version.')
        self.debug('Current version saved.  .')

        # Now increment the version
        self.debug(f'Beginning bump of `{part}`...')
        if part == 'major':
            self.version.bump_major()
        elif part == 'minor':
            self.version.bump_minor()
        elif part == 'patch':
            self.version.bump_patch()
        elif part == 'prerelease':
            self.version.bump_prerelease()
        elif part == 'build':
            self.version.bump_build()
        else:
            raise ValueError(f'unknown part: {part}')
        self.debug(f'+ New value of {part}: {getattr(self.version, part)}.')

        for file in self.source_files:
            file.update(self.version)

        # Write the new version to config file
        self._persist_version()

        if self.new_release:
            # The actual auto_commit and auto_tag
            if self.auto_commit and not self.dry_run:
                self.git.add()
                self.git.commit(f'auto-commit triggered by version bump')

            if self.auto_tag and not self.dry_run:
                msg = f'auto-generated release: {self.version.semantic}'
                self.git.tag(self.tag_name, self.signature, msg)

            if self.auto_push and not self.dry_run:
                self.git.push(self.tag_name)

        return self

==> bumpster-0.1.11.tar.gz: bumpster-0.1.11/bumpster/bump.py <==
# -*- coding: utf-8 -*-

from configparser import ConfigParser
from pathlib import Path

from .color import ColorUI
from .version import Version
from .git import Git
from .source import SourceCode
from . import __version__


#######################################
#
#   Bump Class
#
#######################################


class Bump(object):
    """
    Object which will orchestrate entire program
    """
    CONFIG_FILE = '.bumpster'
    DEFAULT_CONFIG = {
        'version': {'current': '0.1.0'},
        'options': {
            'auto_tag': False,
            'auto_commit': False,
            'auto_push': False,
            'signature': False
        },
        'file1': {
            'file': '*.py',
            'regex': '__version__\s*=\s*(?P<quote>[\'\"])(?P<version>.+)(?P=quote)',
            'style': 'semantic'
        }
    }

    def _read_file(self, f):
        return Path(f).read_text()

    def __init__(self, verbose=False, dry_run=False, defaults=False, config_file=None):
        self.ran_from = Path.cwd()
        self.verbose = verbose
        self.dryrun = dry_run
        self.defaults = defaults
        self.config_file = config_file
        self.filename = None
        self.version = None
        self.bumped_part = None

        self.config = None
        self.config_file = None
        self.auto_sign = None
        self.auto_commit = None
        self.auto_push = None
        self.signature = None
        self.source_files = []
        
        self.debug(f'   ---=== Welcome to bumpster v{__version__} ===---')
        self.debug('Verbose output selected.')
        if self.dryrun:
            self.print('--dry-run detected.  No files will be written to.', ColorUI.CYAN)
        if self.dry_run and not self.verbose:
            self.print('FYI, --dry-run without --verbose is fairly useless.', ColorUI.RED)
        if self.defaults:
            self.debug('Default config selected via --defaults.')
        
        self.debug('Investigating a project git repo.')
        self.git = Git()
        if self.git.usable:
            v = self.git.current_version
            p = self.git.distance_to_latest_tag
            b = self.git.commit_sha
            self.debug(f'Parsed git version {v}-{p}+{b}.')
        self._read_config()
        
        return

    def _read_config(self):
        """
        During the Bump __init__() this method helps read in the config.  We search 
        for a config in several locations.  Also, the user may specify default 
        config, which we load here as well.
        """
        f = self.config_file or Bump.CONFIG_FILE
        candidates = [
            Path(self.ran_from, f),
            Path(self.ran_from.parent, f),
            Path(self.git.root, f),
            Path(self.ran_from.parent.parent, f)
        ]
        
        config = ConfigParser()
        if self.defaults:
            config.read_dict(Bump.DEFAULT_CONFIG)

            file = Path(self.ran_from, Bump.CONFIG_FILE)
            config.write(open(file, "w"))
            self.config_file = file
        else:
            # Attempt to load settings from multiple locations
            for c in candidates:
                if c.exists():
                    config.read(c)
                    self.config_file = c
                    break
            else:
                raise ValueError(f'Could not find config file [{Bump.CONFIG_FILE}].')

        # Loop through config to debug it if --verbose
        for section in config.sections():
            self.debug(f'* Config section parsed: --== {section} ==--')
            for name, value in config.items(section):
                self.debug(f'* {name} = {value}')

        # CURRENT VERSION
        version = config.get('version', 'current', fallback='0.1.0')
        v = Version.parse(version)
        self.version = v
        self.debug(f'Parsed version {v.semantic} from config.')

        # OPTIONS
        self.auto_tag = config.getboolean('options', 'auto-tag', fallback=False)
        self.auto_commit = config.getboolean('options', 'auto-commit', fallback=False)
        self.auto_push = config.getboolean('options', 'auto-push', fallback=False)
        self.signature = config.getboolean('options', 'signature', fallback=False)

        o = 'auto_tag={} auto_commit={} auto_push={} signature={}'.format(
            self.auto_tag,
            self.auto_commit,
            self.auto_push,
            self.signature)
        self.debug(f'Options parsed.  {o}')


        # We need to verify sections are there, especially before we blindly remove

        # FILES TO WRITE
        sections = config.sections().copy()
        sections.remove('version')
        sections.remove('options')
        for s in sections:
            file = config.get(s, 'file', fallback=None)
            regex = config.get(s, 'regex', fallback=None)
            style = config.get(s, 'style', fallback='version')
            
            # First throw an error if need be
            if file is None or regex is None:
                self.debug(f'Skipping {s}, missing file or regex')
                continue

            # Case with *.py for all python files
            if '*' in file:
                files = self.search_for_files(file)
                for f in files:
                    u = SourceCode(self, f, regex, style)
                    self.source_files.append(u)
            # This is the normal case, 1 file per section
            else:
                u = SourceCode(self, file, regex, style)
                self.source_files.append(u)

        self.config = config
        return None

    def _persist_version(self):
        """
        Update the version in the config file then write it so we know the
        new version next time
        """
        self.debug(f'Persisting version {self.version.semantic} in config file.')
        
        self.config.set('version', 'current', self.version.semantic)
        if not self.dry_run:
            self.config.write(open(self.config_file, "w"))

        return None

    @property
    def new_release(self):
        if self.version.prerelease:
            return False

        if self.bumped_part in ['major', 'minor', 'patch']:
            self.debug('New release will trigger git tag, commit, push.')
            return True

    @property
    def config_root(self):
        return self.config_file.parent

    @property
    def dry_run(self):
        if self.dryrun:
            self.print('Skipping file write due to --dry-run', ColorUI.RED)
            return True
        return False

    def print(self, line, color=ColorUI.BLUE):
        """
        Print with fancy colors
        """

        print(f'{color}{line}{ColorUI.ENDC}')

    def debug(self, line):
        """
        Simple wrapper used to output debug messages
        for those using the flag --verbose
        """
        
        if self.verbose:
            self.print(line, ColorUI.PINK)

    def search_for_files(self, f):
        # First we look in current_dir + all subdirs
        root = self.config_root
        self.debug(f'Searching for {f} in dir {root}')
        matches = [x for x in Path(root).rglob(f)]
        if len(matches) > 0:
            return matches

    def parse_version_from_file(self, f):
        """
        Often times the only record of current_version is inside
        a source file such as __init__.py
        """

        file_txt = self._read_file(f)
        self.debug(f'Opened and read file: {f} to memory.')

        match = self.compiled_version.search(file_txt, re.MULTILINE)
        if match is None:
            raise ValueError('Could not find __version__ in __init__.py.')
        self.debug(f'Found potential version: {match.group(1)}.')

        v = Version.parse(match.group(1))
        self.debug(f'Successfully parsed version: {v.version}.')

        return v

    def bump(self, part):
        """
        This is pretty much the full process
        """

        self.bumped_part = part
        self.debug(f'Beginning bump of `{part}`...')
        self.debug('First we must save the current version.')
        self.debug('Current version saved.  .')

        # Now increment the version
        self.debug(f'Beginning bump of `{part}`...')
        if part == 'major':
            self.version.bump_major()
            self.debug(f'+ New value of major: {self.version.major}.')
        elif part == 'minor':
            self.version.bump_minor()
            self.debug(f'+ New value of minor: {self.version.minor}.')
        elif part == 'patch':
            self.version.bump_patch()
            self.debug(f'+ New value of patch: {self.version.patch}.')
        elif part == 'prerelease':
            self.version.bump_prerelease()
            self.debug(f'+ New value of prerelease: {self.version.prerelease}.')
        elif part == 'build':
            self.version.bump_build()
            self.debug(f'+ New value of build: {self.version.build}.')
        else:
            raise ValueError(f'unknown part: {part}')

        for file in self.source_files:
            file.update(self.version)

        if self.new_release:
            # The actual auto_commit and auto_tag
            if self.auto_commit and not self.dry_run:
                self.git.add()
                self.git.commit(f'auto-commit triggered by version bump')
            if self.auto_tag and not self.dry_run:
                msg = f'auto-tag generated release: {self.version.semantic}'
                self.git.tag(self.version.public, self.signature, msg)
            if self.auto_push and not self.dry_run:
                self.git.push(self.version.public)

        # Write the new version to config file
        self._persist_version()

        return self
